@@ -20,43 +20,21 @@ from __future__ import absolute_import
 
 __author__ = "imron@scalyr.com"
 
-import os
 import re
+import ssl
 import socket
-import subprocess
-import sys
-from tempfile import mkstemp
-from io import open
 
 import six
 import six.moves.http_client
 
 import scalyr_agent.scalyr_logging as scalyr_logging
+from scalyr_agent.compat import ssl_match_hostname
+from scalyr_agent.compat import CertificateError
+from scalyr_agent.compat import PY_post_equal_279
 
 
 log = scalyr_logging.getLogger(__name__)
 
-# noinspection PyBroadException
-try:
-    import ssl
-
-    __has_ssl__ = True
-except Exception:
-    __has_ssl__ = False
-    ssl = None  # type: ignore
-
-
-__has_pure_python_tls__ = False
-try:
-    from tlslite import (  # pylint: disable=import-error
-        HTTPTLSConnection,
-        HandshakeSettings,
-    )
-
-    __has_pure_python_tls__ = True
-except Exception:
-    pass
-
 
 class ConnectionFactory:
     """ Factory class for creating connection objects using either Httplib or Requests
@@ -71,7 +49,6 @@ class ConnectionFactory:
         intermediate_certs_file,
         headers,
         use_requests,
-        use_tlslite,
         quiet=False,
         proxies=None,
     ):
@@ -84,7 +61,6 @@ class ConnectionFactory:
         @param intermediate_certs_file: path to certificate bundle for trusted intermediate server certificates
         @param headers: any headers to send with each request made on the connection
         @param use_requests: whether or not to use Requests for handling queries
-        @param use_tlslite: whether or not to use TLSLite for TLS connections
         @param quiet:  Whether or not to emit non-error log messages.
         @param proxies:  A dict describing the network proxies to use or None if there aren't any.  Only valid if
             use_requests is True.
@@ -95,7 +71,6 @@ class ConnectionFactory:
         @type intermediate_certs_file: six.text_type
         @type headers: dict
         @type use_requests: bool
-        @type use_tlslite: bool
         @type quiet: bool
         @type proxies: dict
 
@@ -104,18 +79,6 @@ class ConnectionFactory:
 
         """
 
-        if use_tlslite:
-            if use_requests:
-                log.warn(
-                    "Both `use_requests_lib` and `use_tlslite` are set to True.  `use_tlslite` is ignored if `use_requests_lib` is True"
-                )
-
-            if sys.version_info[:3] >= (2, 7, 9):
-                log.warn(
-                    "`use_tlslite` is only valid when running on Python 2.7.8 or below"
-                )
-                use_tlslite = False
-
         result = None
         if use_requests:
             try:
@@ -130,36 +93,19 @@ class ConnectionFactory:
                     % six.text_type(e)
                 )
                 result = ScalyrHttpConnection(
-                    server,
-                    request_deadline,
-                    ca_file,
-                    intermediate_certs_file,
-                    headers,
-                    use_tlslite,
+                    server, request_deadline, ca_file, intermediate_certs_file, headers,
                 )
                 use_requests = False
         else:
             result = ScalyrHttpConnection(
-                server,
-                request_deadline,
-                ca_file,
-                intermediate_certs_file,
-                headers,
-                use_tlslite,
+                server, request_deadline, ca_file, intermediate_certs_file, headers,
             )
 
         if not quiet:
             if use_requests:
                 log.info("Using Requests for HTTP(S) connections")
             else:
-                if result.is_pure_python_tls:
-                    log.info(
-                        "Using ScalyrHttpConnection/tlslite for HTTP(S) connections"
-                    )
-                else:
-                    log.info(
-                        "Using ScalyrHttpConnection/Httplib for HTTP(S) connections"
-                    )
+                log.info("Using ScalyrHttpConnection/Httplib for HTTP(S) connections")
 
         return result
 
@@ -204,26 +150,15 @@ class Connection(object):
 
     def __check_ssl(self):
         """ Helper function to check if ssl is available and enabled """
-        if self._use_ssl:
-            if not __has_ssl__ and not __has_pure_python_tls__:
-                log.warn(
-                    "No ssl library available so cannot verify server certificate when communicating with Scalyr. "
-                    "This means traffic is encrypted but can be intercepted through a man-in-the-middle attack. "
-                    "To solve this, install the Python ssl library. "
-                    "For more details, see https://www.scalyr.com/help/scalyr-agent#ssl",
-                    limit_once_per_x_secs=86400,
-                    limit_key="nosslwarning",
-                    error_code="client/nossl",
-                )
-            elif self._ca_file is None:
-                log.warn(
-                    "Server certificate validation has been disabled while communicating with Scalyr. "
-                    "This means traffic is encrypted but can be intercepted through a man-in-the-middle attach. "
-                    "Please update your configuration file to re-enable server certificate validation.",
-                    limit_once_per_x_secs=86400,
-                    limit_key="nocertwarning",
-                    error_code="client/sslverifyoff",
-                )
+        if self._use_ssl and not self._ca_file:
+            log.warn(
+                "Server certificate validation has been disabled while communicating with Scalyr. "
+                "This means traffic is encrypted but can be intercepted through a man-in-the-middle attack. "
+                "Please update your configuration file to re-enable server certificate validation.",
+                limit_once_per_x_secs=86400,
+                limit_key="nocertwarning",
+                error_code="client/sslverifyoff",
+            )
 
     def post(self, request_path, body):
         """Post requests"""
@@ -268,19 +203,9 @@ class Connection(object):
         pass
 
 
-class CertValidationError(Exception):
-    pass
-
-
 class ScalyrHttpConnection(Connection):
     def __init__(
-        self,
-        server,
-        request_deadline,
-        ca_file,
-        intermediate_certs_file,
-        headers,
-        use_tlslite,
+        self, server, request_deadline, ca_file, intermediate_certs_file, headers,
     ):
         super(ScalyrHttpConnection, self).__init__(
             server, request_deadline, ca_file, headers
@@ -288,188 +213,17 @@ class ScalyrHttpConnection(Connection):
         self.__http_response = None
         self._intermediate_certs_file = intermediate_certs_file
 
-        try:
-            self._init_connection(pure_python_tls=use_tlslite)
-        except Exception:  # echee TODO: more specific exception representative of TLS1.2 incompatibility
-            log.info(
-                "Exception while attempting to init HTTP Connection.  Falling back to pure-python TLS implementation"
-            )
-            self._init_connection(pure_python_tls=True)
+        self._init_connection()
 
-        if self.is_pure_python_tls:
-            log.info("HttpConnection uses pure-python TLS")
-        else:
-            log.info("HttpConnection uses native os ssl")
+        log.info("HttpConnection uses native os ssl")
 
-    def _validate_chain_openssl(self):
-        """Validate server certificate chain using openssl system callout"""
-        # fetch end-entity certificate and write to tempfile
-        end_entity_pem = ssl.get_server_certificate((self._host, self._port))
-        try:
-            end_entity_pem_tempfile_fd, end_entity_pem_tempfile_path = mkstemp()
-            # NOTE: We close the fd here because we open it again below. This way file deletion at
-            # the end works correctly on Windows.
-            os.close(end_entity_pem_tempfile_fd)
-
-            with os.fdopen(end_entity_pem_tempfile_fd, "w") as fp:
-                fp.write(end_entity_pem)
-
-            # invoke openssl
-            # root_certs = '/usr/share/scalyr-agent-2/certs/ca_certs.crt'
-            # intermediate_certs = '/usr/share/scalyr-agent-2/certs/intermediate_certs.pem'
-            cmd = [
-                "openssl",
-                "verify",
-                "-CAfile",
-                self._ca_file,
-                "-untrusted",
-                self._intermediate_certs_file,
-                end_entity_pem_tempfile_path,
-            ]
-            log.debug("Validating server certificate chain via: %s" % cmd)
-            proc = subprocess.Popen(
-                args=" ".join(cmd),
-                stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE,
-                shell=True,
-            )
-            out, err = proc.communicate()
-            returncode = proc.returncode
-            if returncode != 0:
-                raise CertValidationError(err.strip())
-            log.info("Scalyr server cert chain successfully validated via openssl")
-        finally:
-            # delete temp file
-            os.remove(end_entity_pem_tempfile_path)
-
-    def _validate_chain_certvalidator(self, tlslite_connection):
-        """Validate server certificate chain using 3rd party certvalidator library which uses oscrypt/libcrypto
-        Note: oscrypt uses ctypes find_library() which does not work in certain distributions such as alpine.
-        (e.g. see https://github.com/docker-library/python/issues/111)
-        On such systems, users will have to rely on other server cert validation approaches such as using openssl
-        or turning it off completely.
-        """
-        try:
-            # pylint: disable=import-error,no-name-in-module
-            from certvalidator import CertificateValidator
-            from certvalidator import ValidationContext
-            from asn1crypto import (
-                x509,
-                pem,
-            )
-
-            # pylint: enable=import-error,no-name-in-module
-
-            # validate server certificate chain
-            session = tlslite_connection.sock.session
-            assert type(session.serverCertChain.x509List) == list
-
-            # get the end-entity cert
-            file_bytes = session.serverCertChain.x509List[0].bytes
-            end_entity_cert = x509.Certificate.load(six.binary_type(file_bytes))
-
-            def cert_files_exist(path, file_names):
-                file_names = [os.path.join(path, f) for f in file_names]
-                for f in file_names:
-                    if not os.path.isfile(f):
-                        return False
-                return True
-
-            def get_cert_bytes(cert_dir, file_names):
-                file_names = [os.path.join(cert_dir, f) for f in file_names]
-                result = []
-                for fname in file_names:
-                    arr = open(fname, "rb").read()
-                    cert_bytes = pem.unarmor(arr)[2]
-                    result.append(cert_bytes)
-                return result
-
-            intermediate_cert_names = [
-                "comodo_ca_intermediate.pem",
-                "sectigo_ca_intermediate.pem",
-            ]
-
-            extra_trust_names = [
-                "scalyr_agent_ca_root.pem",
-                "addtrust_external_ca_root.pem",
-            ]
-
-            # Determine the directory containing the certs.
-            # First check the directory containing the _ca_file
-            # but if we don't find the intermediate/extra certs there
-            # then look in the relative `certs` directory.  The latter
-            # will typically be required if running directly from source
-            all_cert_names = intermediate_cert_names + extra_trust_names
-            cert_dir = os.path.dirname(self._ca_file)
-            if not cert_files_exist(cert_dir, all_cert_names):
-                path = os.path.dirname(os.path.abspath(__file__))
-                path = os.path.abspath(path + "../../certs")
-                if cert_files_exist(path, all_cert_names):
-                    cert_dir = path
-
-            trust_roots = None
-            intermediate_certs = get_cert_bytes(cert_dir, intermediate_cert_names)
-            extra_trust_roots = get_cert_bytes(cert_dir, extra_trust_names)
-
-            if trust_roots:
-                context = ValidationContext(
-                    trust_roots=trust_roots,
-                    extra_trust_roots=extra_trust_roots,
-                    other_certs=intermediate_certs,
-                    # whitelisted_certs=[end_entity_cert.sha1_fingerprint],
-                )
-            else:
-                context = ValidationContext(
-                    extra_trust_roots=extra_trust_roots,
-                    other_certs=intermediate_certs,
-                    # whitelisted_certs=[end_entity_cert.sha1_fingerprint],
-                )
-                validator = CertificateValidator(
-                    end_entity_cert, validation_context=context
-                )
-            validator.validate_tls(six.text_type(self._host))
-            log.info(
-                "Scalyr server cert chain successfully validated via certvalidator library"
-            )
-        except Exception as ce:
-            log.exception("Error validating server certificate chain: %s" % ce)
-            raise
-
-    def _init_connection(self, pure_python_tls=False):
+    def _init_connection(self):
         try:
             if self._use_ssl:
-                if not pure_python_tls:
-                    # If we do not have the SSL library, then we cannot do server certificate validation anyway.
-                    if __has_ssl__:
-                        ca_file = self._ca_file
-                    else:
-                        ca_file = None
-                    self.__connection = HTTPSConnectionWithTimeoutAndVerification(
-                        self._host,
-                        self._port,
-                        self._request_deadline,
-                        ca_file,
-                        __has_ssl__,
-                    )
-                    self.__connection.connect()
-                else:
-                    # Pure python implementation of TLS does not require ssl library
-                    settings = HandshakeSettings()
-                    settings.minVersion = (3, 3)  # TLS 1.2
-                    self.__connection = HTTPTLSConnection(
-                        self._host,
-                        self._port,
-                        timeout=self._request_deadline,
-                        settings=settings,
-                    )
-                    self.__connection.connect()
-                    # Non-null _ca_file signifies server cert validation is requireds
-                    if self._ca_file:
-                        try:
-                            self._validate_chain_certvalidator(self.__connection)
-                        except Exception:
-                            log.exception("Failure in _validate_chain_certvalidator()")
-                            self._validate_chain_openssl()
+                self.__connection = HTTPSConnectionWithTimeoutAndVerification(
+                    self._host, self._port, self._request_deadline, self._ca_file,
+                )
+                self.__connection.connect()
             else:
                 # unencrypted connection
                 self.__connection = HTTPConnectionWithTimeout(
@@ -481,15 +235,21 @@ class ScalyrHttpConnection(Connection):
                 errno = error.errno  # pylint: disable=no-member
             else:
                 errno = None
-            if isinstance(error, CertValidationError):
-                log.error(
-                    'Failed to connect to "%s" because of server certificate validation error: "%s"',
+
+            error_code = "client/connectionFailed"
+
+            if isinstance(error, CertificateError):
+                error_code = "client/connectionFailedCertHostnameValidationFailed"
+                log.exception(
+                    'Failed to connect to "%s" because of server certificate validation error: "%s". '
+                    "This likely indicates a MITM attack.",
                     self._full_address,
                     getattr(error, "message", str(error)),
-                    error_code="client/connectionFailed",
+                    error_code=error_code,
                 )
-            elif __has_ssl__ and isinstance(error, ssl.SSLError):
-                log.error(
+            elif isinstance(error, ssl.SSLError):
+                error_code = "client/connectionFailedSSLError"
+                log.exception(
                     'Failed to connect to "%s" due to some SSL error.  Possibly the configured certificate '
                     "for the root Certificate Authority could not be parsed, or we attempted to connect to "
                     "a server whose certificate could not be trusted (if so, maybe Scalyr's SSL cert has "
@@ -498,38 +258,46 @@ class ScalyrHttpConnection(Connection):
                     self._full_address,
                     errno,
                     six.text_type(error),
-                    error_code="client/connectionFailed",
+                    error_code=error_code,
                 )
             elif errno == 61:  # Connection refused
-                log.error(
+                error_code = "client/connectionFailedConnRefused"
+                log.exception(
                     'Failed to connect to "%s" because connection was refused.  Server may be unavailable.',
                     self._full_address,
-                    error_code="client/connectionFailed",
+                    error_code=error_code,
                 )
             elif errno == 8:  # Unknown name
-                log.error(
+                error_code = "client/connectionFailed"
+                log.exception(
                     'Failed to connect to "%s" because could not resolve address.  Server host may be bad.',
                     self._full_address,
-                    error_code="client/connectionFailed",
+                    error_code=error_code,
                 )
             elif errno is not None:
-                log.error(
+                error_code = "client/connectionFailed"
+                log.exception(
                     'Failed to connect to "%s" due to errno=%d.  Exception was "%s".  Closing connection, '
                     "will re-attempt",
                     self._full_address,
                     errno,
                     six.text_type(error),
-                    error_code="client/connectionFailed",
+                    error_code=error_code,
                 )
             else:
-                log.error(
+                error_code = "client/connectionFailed"
+                log.exception(
                     'Failed to connect to "%s" due to exception.  Exception was "%s".  Closing connection, '
                     "will re-attempt",
                     self._full_address,
                     six.text_type(error),
-                    error_code="client/connectionFailed",
+                    error_code=error_code,
                 )
-            raise Exception("client/connectionFailed")
+
+            # TODO: We should probably propagate original exception class...
+            exc = Exception("%s. Original error: %s" % (error_code, str(error)))
+            exc.error_code = error_code
+            raise exc
 
     # 2->TODO ensure that httplib request accepts only binary data, even method name and
     #  request path python2 httplib can not handle mixed data
@@ -565,12 +333,6 @@ class ScalyrHttpConnection(Connection):
     def close(self):
         self.__connection.close()
 
-    @property
-    def is_pure_python_tls(self):
-        return __has_pure_python_tls__ and isinstance(
-            self.__connection, HTTPTLSConnection
-        )
-
 
 class HTTPConnectionWithTimeout(six.moves.http_client.HTTPConnection):
     """An HTTPConnection replacement with added support for setting a timeout on all blocking operations.
@@ -607,7 +369,7 @@ class HTTPSConnectionWithTimeoutAndVerification(six.moves.http_client.HTTPSConne
     class implements the necessary support.
     """
 
-    def __init__(self, host, port, timeout, ca_file, has_ssl):
+    def __init__(self, host, port, timeout, ca_file):
         """
         Creates an instance.
 
@@ -616,40 +378,16 @@ class HTTPSConnectionWithTimeoutAndVerification(six.moves.http_client.HTTPSConne
             port: The port to connect to.
             timeout: The timeout, in seconds, to use for all blocking operations on the underlying socket.
             ca_file:  If not None, then this is a file containing the certificate authority's root cert to use
-                for validating the certificate sent by the server.  This must be None if has_ssl is False.
+                for validating the certificate sent by the server.
                 If None is passed in, then no validation of the server certificate will be done whatsoever, so
                 you will be susceptible to man-in-the-middle attacks.  However, at least your traffic will be
                 encrypted.
-            has_ssl:  True if the ssl Python library is available.
         """
-        if not has_ssl and ca_file is not None:
-            raise Exception(
-                "If has_ssl is false, you are not allowed to specify a ca_file because it has no affect."
-            )
         self.__timeout = timeout
         self.__ca_file = ca_file
-        self.__has_ssl = has_ssl
         six.moves.http_client.HTTPSConnection.__init__(self, host, port)
 
     def connect(self):
-        # Do not delete the next line:
-        # SIMULATE_TLS12_FAILURE raise Exception('Fake a failed connection with ssl lib')
-
-        # If the ssl library is not available, then we just have to fall back on old HTTPSConnection.connect
-        # method.  There are too many dependencies to implement it directly here.
-        if not self.__has_ssl:
-            # Unfortunately, the only way to set timeout is to temporarily set the global default timeout
-            # for what it should be for this connection, and then reset it once the connection is established.
-            # Messy, but not much we can do.
-            old_timeout = None
-            try:
-                old_timeout = socket.getdefaulttimeout()
-                socket.setdefaulttimeout(self.__timeout)
-                six.moves.http_client.HTTPSConnection.connect(self)
-                return
-            finally:
-                socket.setdefaulttimeout(old_timeout)
-
         # Create the underlying socket.  Prefer Python's newer socket.create_connection method if it is available.
         if hasattr(socket, "create_connection"):
             self.sock = socket.create_connection((self.host, self.port), self.__timeout)
@@ -662,11 +400,72 @@ class HTTPSConnectionWithTimeoutAndVerification(six.moves.http_client.HTTPSConne
             self._tunnel()
 
         # Now ask the ssl library to wrap the socket and verify the server certificate if we have a ca_file.
-        if self.__ca_file is not None:
-            self.sock = ssl.wrap_socket(
-                self.sock, ca_certs=self.__ca_file, cert_reqs=ssl.CERT_REQUIRED
-            )
+        if self.__ca_file:
+            if PY_post_equal_279:
+                # ssl.PROTOCOL_TLSv1_2 was added in Python 2.7.9 so we request that protocol if we
+                # are running on that or newer version. Since July 2020, Scalyr API side now only
+                # supports TLSv1.2.
+                # On newer versions we also use a more secure SSLContext which gives us more flexibility
+                # and control over the options vs using ssl.wrap_socket on older versions
+                ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
+                ssl_context.options |= ssl.OP_NO_SSLv2
+                ssl_context.options |= ssl.OP_NO_SSLv3
+                ssl_context.options |= ssl.OP_NO_TLSv1
+                ssl_context.options |= ssl.OP_NO_TLSv1_1
+                ssl_context.verify_mode = ssl.CERT_REQUIRED
+                ssl_context.check_hostname = True
+                ssl_context.load_verify_locations(self.__ca_file)
+
+                self.sock = ssl_context.wrap_socket(
+                    self.sock, do_handshake_on_connect=True, server_hostname=self.host
+                )
+
+                # Additional asserts / guards
+                assert (
+                    self.sock._context.verify_mode == ssl.CERT_REQUIRED
+                ), "invalid verify_mode"
+                assert (
+                    self.sock._context.check_hostname is True
+                ), "check_hostname is False"
+                # NOTE: By default we use our bundled ca certs file with two CA certs, but user
+                # could specify to use a system bundle we can't assert on number being exactly
+                # 2
+                assert (
+                    len(self.sock._context.get_ca_certs()) >= 1
+                ), "ca certs not loaded"
+                assert (
+                    self.sock.do_handshake_on_connect is True
+                ), "do_handshake_on_connect is false"
+            else:
+                # server_hostname argument was added in 2.7.9 so before that version we won't send
+                # SNI and also use ssl.wrap_socket instead of ssl.SSLContext
+                self.sock = ssl.wrap_socket(
+                    self.sock, ca_certs=self.__ca_file, cert_reqs=ssl.CERT_REQUIRED
+                )
+
+                # Additional asserts / guards
+                assert self.sock.ca_certs, "ca_certs is falsy"
+                assert self.sock.cert_reqs == ssl.CERT_REQUIRED, "cert_reqs is invalid"
+                assert (
+                    self.sock.do_handshake_on_connect is True
+                ), "do_handshake_on_connect is false"
+
+            # NOTE: In newer versions of Python when using SSLContext + check_hostname = True, this
+            # should happen automatically post handshake, but to also support older versions and
+            # ensure this verification is always performed, we call this method manually here.
+            # Will throws CertificateError in case host validation fails
+            cert = self.sock.getpeercert()
+            ssl_match_hostname(cert=cert, hostname=self.host)
         else:
+            log.warn(
+                "Server certificate validation has been disabled while communicating with Scalyr. "
+                "This means traffic is encrypted but can be intercepted through a man-in-the-middle attack. "
+                "Please update your configuration file to re-enable server certificate validation.",
+                limit_once_per_x_secs=86400,
+                limit_key="nocertwarning",
+                error_code="client/sslverifyoff",
+            )
+
             self.sock = ssl.wrap_socket(self.sock, cert_reqs=ssl.CERT_NONE)
 
 
